#![warn (clippy::pedantic)]

pub mod graceful_shutdown;
pub mod http_serde;
pub mod prelude;

// It's easier if the server can stream its response body
// back to the relay un-changed inside its request body
// So we wrap the server's actual response head
// (status code, headers, etc.) in this one header field.

pub const PTTH_MAGIC_HEADER: &str = "X-PTTH-2LJYXWC4";

// The arguments are in order so they are in order overall:
// e.g. prefix_match ("/prefix", "/prefix/middle/suffix") -> "/middle/suffix"

#[must_use]
pub fn prefix_match <'a> (prefix: &str, hay: &'a str) -> Option <&'a str>
{
	if hay.starts_with (prefix) {
		Some (&hay [prefix.len ()..])
	}
	else {
		None
	}
}

#[cfg (test)]
mod tests {
	use super::*;
	#[test]
	fn prefix () {
		for (p, h, expected) in &[
			("/files/", "/files/a", Some ("a")),
			("/files/", "/files/abc/def", Some ("abc/def")),
			("/files/", "/files", None),
			("/files/", "/not_files", None),
			("/files/", "/files/", Some ("")),
		] {
			assert_eq! (prefix_match (*p, *h), *expected);
		}
	}
}
